import {Between, EntityRepository, FindOperator, getConnection, Repository} from "typeorm";
import {UpdateOrderDto} from "./dto/update-order.dto";
import {Order} from "./entity/orders.entity";

@EntityRepository(Order)
export class OrderRepository extends Repository<Order> {

    async createEntity(dto: any): Promise<any | Error> {
        try {
            const entity = this.create(dto);
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async countOrder(): Promise<any | Error> {
        try {
            return await this.count({
                where: {isDeleted: false}
            });
        } catch (e) {
            return e;
        }
    }
    async countOrderWithDeliveryStatus(status: string): Promise<any | Error> {
        try {
            return await this.count({
                where: {isDeleted: false, status: status}
            });
        } catch (e) {
            return e;
        }
    }
    async countUserOrderStatus(id: string, status: string): Promise<any | Error> {
        try {
            return await this.count({
                where: {isDeleted: false, status: status, customerId: id}
            });
        } catch (e) {
            return e;
        }
    }
    async countDeliveredOrder(status: string): Promise<any | Error> {
        try {
            return await this.count({
                where: {isDeleted: false, status: status}
            });
        } catch (e) {
            return e;
        }
    }

    async countCancelledOrder(status: string): Promise<any | Error> {
        try {
            return await this.count({
                where: {isDeleted: false, status: status}
            });
        } catch (e) {
            return e;
        }
    }

    async countPendingOrder(status: string): Promise<any | Error> {
        try {
            return await this.count({
                where: {isDeleted: false, status: status}
            });
        } catch (e) {
            return e;
        }
    }

    async findAllEntities(take: number, page: number, query: any): Promise<any | Error> {
        try {
            let strstartDate: any;
            let strendDate: any;
            type FindOption = {
                paymentStatus?: FindOperator<string>,
                status?: FindOperator<string>,
                createdAt?: FindOperator<Date>,
                customerName?: FindOperator<string>,
                orderId?: FindOperator<string>,
            }
            const findOptions: FindOption = {};

            if (query.paymentStatus) findOptions.paymentStatus = query.paymentStatus;
            if (query.deliveryStatus) findOptions.status = query.deliveryStatus;
            if (query.customerName) findOptions.customerName = query.customerName;
            if (query.orderId) findOptions.orderId = query.orderId;
            if (query.startDate) strstartDate = query.startDate;
            if (query.endDate) strendDate = query.endDate;

            const startDate = new Date(strstartDate);
            startDate.setUTCHours(0, 0, 0, 0);
            const endDate = new Date(strendDate);
            endDate.setUTCHours(23, 59, 59, 999);

            if (query.startDate && query.endDate) findOptions.createdAt = Between(
                new Date(startDate),
                new Date(endDate),
            );

            const data = await this.find({
                take, skip: take * (page - 1),
                order: {createdAt: "DESC"},
                where: {
                    ...findOptions,
                    isDeleted: false
                },
                // relations: ["orders"]
            });

            const totalOrders = await this.count({
                where: {isDeleted: false}
            });
            const totalDeliveredOrders = await this.count({
                where: {isDeleted: false, status: 'Delivered'}
            });
            const totalCancelledOrders = await this.count({
                where: {isDeleted: false, status: 'Cancelled'}
            });
            const totalPendingOrders = await this.count({
                where: {isDeleted: false, status: 'Pending'}
            });
            const responseData = {
                data: data,
                totalOrders: totalOrders,
                totalDeliveredOrders: totalDeliveredOrders,
                totalCancelledOrders: totalCancelledOrders,
                totalPendingOrders: totalPendingOrders
            };

            return responseData;
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(
        id: string
    ): Promise<Order[] | Error> {
        let query: any = {orderId: id};
        query.isDeleted = false;

        try {
            return await this.find({
                    where: query,
                    relations: ['orders'],
                },
            );
        } catch (e) {
            return e;
        }
    }

    async findOneEntityWihoutRelation(
        id: string
    ): Promise<Order[] | Error> {
        let query: any = {orderId: id};
        query.isDeleted = false;

        try {
            return await this.find({
                    where: query,
                },
            );
        } catch (e) {
            return e;
        }
    }


    async updateOrder(
        id: string,
        dto: UpdateOrderDto
    ): Promise<Order | Error> {
        try {
            await this.update(id, {
                paymentStatus: dto.paymentStatus,
                status: dto.status
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateOrderStatus(
        id: string,
        status: string
    ): Promise<any | Error> {
        try {
            await this.update(id, {
                status: status
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findSearchedOrder(
        limit: number,
        page: number,
        orderId?: string,
        customerId?: String,
    ): Promise<any | Error> {

        try {
            let findOptions: any = {}

            if (orderId) findOptions.orderId = orderId;
            if (customerId) findOptions.customerId = customerId;

            const count = await this.find({where: {...findOptions, isDeleted: false}})
            const pageCount = Math.ceil(count.length / limit);
            const orders = await this.find(
                {
                    where: {...findOptions, isDeleted: false},
                    relations: ["orders"],
                    take: limit,
                    skip: limit * (page - 1),
                }
            );

            return {
                page: page,
                limit: limit,
                pageCount: pageCount,
                orders
            }
        } catch (e) {
            return e;
        }
    }


    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(Order)
                .set({isDeleted: true})
                .where("orderId = :orderId", {orderId: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }

    async countUserOrder(id: string): Promise<any | Error> {
        try {
            return await this.count({
                where: {customerId: id, isDeleted: false}
            });
        } catch (e) {
            return e;
        }
    }

    async recentUserOrderList(id: string): Promise<any | Error> {
        try {
            return await this.find({
                take: 6,
                order: {createdAt: -1},
                where: {customerId: id, isDeleted: false}
            });


        } catch (e) {
            return e;
        }
    }

    async totalUserOrderList(limit: number, page: number, id: string): Promise<any | Error> {
        try {
            let findOptions: any = {}

            if (id) findOptions.customerId = id;

            const count = await this.find({where: {...findOptions, isDeleted: false}})
            const pageCount = Math.ceil(count.length / limit);
            const data = await this.find({
                take: limit,
                skip: limit * (page - 1),
                where: {customerId: id, isDeleted: false}
            });

            return {
                page: page,
                limit: limit,
                pageCount: pageCount,
                data
            }
        } catch (e) {
            return e;
        }
    }

    async countUserDeliveredOrder(id: string, status: string): Promise<any | Error> {
        try {
            return await this.count({
                where: {isDeleted: false, status: status, customerId: id}
            });
        } catch (e) {
            return e;
        }
    }

    async totalUserDeliveredOrderList(limit: number, page: number, id: string, status: string): Promise<any | Error> {
        try {
            let findOptions: any = {}

            // if(orderId) findOptions.orderId = orderId ;
            if (id) findOptions.customerId = id;

            const count = await this.find({where: {...findOptions, isDeleted: false}})
            const pageCount = Math.ceil(count.length / limit);
            const data = await this.find({
                take: limit,
                skip: limit * (page - 1),
                where: {isDeleted: false, status: status, customerId: id}
            });
            return {
                page: page,
                limit: limit,
                pageCount: pageCount,
                data
            }
        } catch (e) {
            return e;
        }
    }

    async checkOrderCount(orderID: string): Promise<any | Error> {
        try {
            return await this.count({
                where: {
                    isDeleted: false, 
                    orderId: orderID
                }
            });
        } catch (e) {
            //return e;
            return 0;
        }
    }

    async countUserCancelledOrder(id: string, status: string): Promise<any | Error> {
        try {
            return await this.count({
                where: {isDeleted: true, customerId: id}
            });
        } catch (e) {
            return e;
        }
    }

    async totalUserCancelledOrderList(limit: number, page: number, id: string, status: string): Promise<any | Error> {
        try {
            let findOptions: any = {}

            // if(orderId) findOptions.orderId = orderId ;
            if (id) findOptions.customerId = id;

            const count = await this.find({where: {...findOptions, isDeleted: false}})
            const pageCount = Math.ceil(count.length / limit);
            const data = await this.find({
                take: limit,
                skip: limit * (page - 1),
                where: {isDeleted: true, customerId: id}
            });

            return {
                page: page,
                limit: limit,
                pageCount: pageCount,
                data
            }
        } catch (e) {
            return e;
        }
    }

    async countUserShippedOrder(id: string, status: string): Promise<any | Error> {
        try {
            return await this.count({
                where: {isDeleted: false, status: status, customerId: id}
            });
        } catch (e) {
            return e;
        }
    }

    async totalUserShippedOrderList(limit: number, page: number, id: string, status: string): Promise<any | Error> {
        try {
            let findOptions: any = {}

            // if(orderId) findOptions.orderId = orderId ;
            if (id) findOptions.customerId = id;

            const count = await this.find({where: {...findOptions, isDeleted: false}})
            const pageCount = Math.ceil(count.length / limit);
            const data = await this.find({
                take: limit,
                skip: limit * (page - 1),
                where: {isDeleted: false, status: status, customerId: id}
            });

            return {
                page: page,
                limit: limit,
                pageCount: pageCount,
                data
            }
        } catch (e) {
            return e;
        }
    }

    async countUserPendingOrder(id: string, status: string): Promise<any | Error> {
        try {
            return await this.count({
                where: {isDeleted: false, status: status, customerId: id}
            });
        } catch (e) {
            return e;
        }
    }

    async totalUserPendingOrderList(limit: number, page: number, id: string, status: string): Promise<any | Error> {
        try {
            let findOptions: any = {}

            // if(orderId) findOptions.orderId = orderId ;
            if (id) findOptions.customerId = id;

            const count = await this.find({where: {...findOptions, isDeleted: false}})
            const pageCount = Math.ceil(count.length / limit);
            const data = await this.find({
                take: limit,
                skip: limit * (page - 1),
                where: {isDeleted: false, status: status, customerId: id}
            });

            return {
                page: page,
                limit: limit,
                pageCount: pageCount,
                data
            }
        } catch (e) {
            return e;
        }
    }

    async totalUserProcessingOrderList(limit: number, page: number, id: string, status: string): Promise<any | Error> {
        try {
            let findOptions: any = {}

            // if(orderId) findOptions.orderId = orderId ;
            if (id) findOptions.customerId = id;

            const count = await this.find({where: {...findOptions, isDeleted: false}})
            const pageCount = Math.ceil(count.length / limit);
            const data = await this.find({
                take: limit,
                skip: limit * (page - 1),
                where: {isDeleted: false, status: status, customerId: id}
            });

            return {
                page: page,
                limit: limit,
                pageCount: pageCount,
                data
            }
        } catch (e) {
            return e;
        }
    }

    async findOrderIdEntity(): Promise<any | Error> {
        const data = await this.find({
            select: ["orderId"]
        });
        return data;
    }

    async hardDeleteEntity(id: string, orderId: string): Promise<boolean | Error> {
        console.log("Delete ID = ", id)
        console.log("Delete Invoice ID = ", orderId)
        try {
            await getConnection()
                .createQueryBuilder()
                .delete()
                .from("order-details")
                .where("orderId = :orderId", {orderId: orderId})
                .execute();

            await getConnection()
                .createQueryBuilder()
                .delete()
                .from(Order)
                .where("id = :id", {id: id})
                .execute();

            return true;
        } catch (e) {
            return e;
        }
    }
}