import { Between, EntityRepository, getConnection, getManager, getRepository, ILike, MoreThan, Repository } from "typeorm";
import { Product } from "./entity/product.entity";


@EntityRepository(Product)
export class ProductsRepository extends Repository<Product> {

    async createEntity(
        dto: any
    ): Promise<any | Error> {
        try {
            const entity = this.create(dto);
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async updateEntity(
        id: string,
        dto: any
    ): Promise<Product | Error> {
        try {
            await this.update(id, dto);
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findAllEntities(take: number, page: number): Promise<Product[] | Error> {
        try {
            return await this.find(
                {
                    take, skip: take * (page - 1),
                    order: { createdAt: "DESC" },
                    where: { isDeleted: false }
                }
            );
        } catch (e) {
            return e;
        }
    }

    async findOneRawEntity(
        id: string
    ): Promise<any | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return false;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(Product)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }

    async findOneProductBySlug(
        slug: string
    ): Promise<Product | Error> {
        let query: any = { urlSlug: slug };
        query.isDeleted = false;
        query.stock = MoreThan(0);
        try {
            return await this.findOne({
                where: query
            });
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<Product[] | Error> {
        try {
            return await this.find({
                order: { createdAt: "DESC" },
                where: { ...query, isDeleted: false }
            });
        }
        catch (e) {
            return e;
        }
    }

    async findAllProducts(
        take: any,
        page: any
    ): Promise<any | Error> {
        try {
            let whereQuery: any = ``;
            let orderByQuery: any = ``;

            if (whereQuery.length > 3) {
                whereQuery += ` and p.isDeleted = 0 and p.isDeactive = 0 `;
            } else {
                whereQuery += ` p.isDeleted = 0 and p.isDeactive = 0 `;
            }

            if (whereQuery.substring(0, 3) === 'and') {
                whereQuery = whereQuery.slice(4, whereQuery.length);
            }

            let conditionQuery = ``;

            if (!whereQuery) {
                conditionQuery = `WHERE p.stock > 0`;
            } else {
                conditionQuery = ` WHERE ${whereQuery} AND p.stock > 0`;
            }

            let offset = 0;

            if (page > 1) {
                offset = ((page - 1) * take);
            }

            let sql = `SELECT p.*, 
            (SELECT GROUP_CONCAT(pcv.colorCode) AS colorCode
            FROM product_color_variants AS pcv
            WHERE pcv.productId = p.id) AS colorCode,
            (SELECT GROUP_CONCAT(pcv.colorName) AS colorName
            FROM product_color_variants AS pcv
            WHERE pcv.productId = p.id) AS colorName FROM products AS p
            LEFT JOIN product_color_variants AS pcv ON p.id = pcv.productId
            ${conditionQuery} GROUP BY p.id ${orderByQuery} limit ${offset}, ${take};`

            let sqlTotal = `SELECT COUNT(*) FROM products as p 
            LEFT JOIN product_color_variants as pcv ON p.id = pcv.productId
            ${conditionQuery} GROUP BY p.id;`

            const entityManager = getManager();
            const data = await entityManager.query(sql);
            const totalData = await entityManager.query(sqlTotal);
            const count = totalData.length;
            const pageCount = Math.ceil(count / take);

            const responseData = {
                page: page,
                limit: take,
                pageCount: pageCount,
                totalDataFound: count,
                data
            };
            return responseData;
        } catch (e) {
            return e;
        }
    }

    async countProduct(): Promise<any | Error> {
        try {
            return await this.count({
                where: { isDeleted: false }
            });
        } catch (e) {
            return e;
        }
    }

    async todayProducts(): Promise<any | Error> {
        const start = new Date();
        start.setUTCHours(0, 0, 0, 0);
        const end = new Date();
        end.setUTCHours(23, 59, 59, 999);

        const data = await this.find({
            where: {
                isDeleted: false,
                createdAt: Between(
                    new Date(start).toISOString(),
                    new Date(end).toISOString()
                )
            },
        });

        return data;
    }

    async topProducts(): Promise<any | Error> {
        try {
            const productData = await this.find({
                where: {
                    isDeleted: false
                },
                order: {
                    totalOrder: -1
                },
                take: 3
            });
            let labelArr = [];
            let dataArr = [];
            let dataArray = [];
            for (let i = 0; i < productData.length; i++) {
                const data = productData[i];
                labelArr.push(data.name);
                dataArr.push(data.totalOrder);
            }
            return {
                labels: labelArr,
                data: dataArr
            };
        } catch (e) {
            return e;
        }
    }

    async getProductsByQuery(productIds: any): Promise<Product[] | Error> {
        try {
            const products = await getConnection()
                .createQueryBuilder()
                .from(Product, "product")
                .where("product.id IN (:...ids)", { ids: productIds })
                .execute();

            return products;
        } catch (e) {
            return e;
        }
    }

    async searchProductByName(name: any): Promise<Product[] | Error> {
        try {
            return await this.find({
                where: {
                    name: ILike(`%${name}%`),
                    isDeleted: 0,
                    stock: MoreThan(0)
                },
            });
        }
        catch (e) {
            return e;
        }
    }
}